package com.example.mdcpoc

import kotlinx.coroutines.delay
import kotlinx.coroutines.slf4j.MDCContext
import kotlinx.coroutines.withContext
import org.slf4j.LoggerFactory
import org.slf4j.MDC
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RestController

@RestController
class TestController {

    companion object {
        val logger = LoggerFactory.getLogger(this::class.java)
    }

    @GetMapping("/test4")
    suspend fun main() {
        logger.info("controller method")
        withContext(MDCContext()) {
            logger.info("Inside coroutine: ${MDC.getCopyOfContextMap()}")

            delay(100)

            logger.info("After suspension: ${MDC.getCopyOfContextMap()}")
            testSuspendFunc()

        }
    }

    suspend fun testSuspendFunc() {
        logger.info("testSuspendFunc started")
        printMDC()
        forLoop()
    }

    private fun printMDC() {
        val copyOfContextMap = MDC.getCopyOfContextMap()
        logger.info("MDC map $copyOfContextMap")
    }

    private suspend fun forLoop() {
        for (i in 1..3) {
            logger.info("Loop delay")
            delay(500L * i)
        }
    }

}
